/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import { renderHook } from '@testing-library/react-hooks';
import { useGetIlmPolicies } from './use_get_ilm_policies';
import * as hookPolicyAPI from './api';

describe('useGetIlmPolicies', () => {
  beforeAll(() => {
    const { policiesData, indexSize } = getTestData();

    jest
      .spyOn(hookPolicyAPI, 'getIlmPolicies')
      .mockReturnValue(new Promise((resolve) => resolve(policiesData)));

    jest
      .spyOn(hookPolicyAPI, 'getIndicesData')
      .mockReturnValue(new Promise((resolve) => resolve(indexSize)));
  });

  it('returns the correct data', async () => {
    const { result, waitForNextUpdate } = renderHook(() => useGetIlmPolicies());
    await waitForNextUpdate();
    expect(result.current.data).toEqual([
      {
        currentSize: '434 MB',
        label: 'All Checks',
        name: 'synthetics',
        policy: policiesData[0],
        retentionPeriod: '180 days + rollover',
      },
      {
        currentSize: '55 MB',
        label: 'Browser Checks',
        name: 'synthetics-synthetics.browser-default_policy',
        retentionPeriod: '365 days + rollover',
        policy: policiesData[1],
      },
      {
        currentSize: '322 MB',
        label: 'Browser Network Requests',
        name: 'synthetics-synthetics.browser_network-default_policy',
        retentionPeriod: '14 days + rollover',
        policy: policiesData[2],
      },
      {
        currentSize: '21 MB',
        label: 'Browser Screenshots',
        name: 'synthetics-synthetics.browser_screenshot-default_policy',
        retentionPeriod: '14 days + rollover',
        policy: policiesData[3],
      },
      {
        currentSize: '36 MB',
        label: 'HTTP Pings',
        name: 'synthetics-synthetics.http-default_policy',
        retentionPeriod: '365 days + rollover',
        policy: policiesData[4],
      },
      {
        currentSize: '0 Bytes',
        label: 'ICMP Pings',
        name: 'synthetics-synthetics.icmp-default_policy',
        retentionPeriod: '365 days + rollover',
        policy: policiesData[5],
      },
      {
        currentSize: '0 Bytes',
        label: 'TCP Pings',
        name: 'synthetics-synthetics.tcp-default_policy',
        retentionPeriod: '365 days + rollover',
        policy: policiesData[6],
      },
    ]);
  });
});

const policiesData: any = [
  {
    name: 'synthetics',
    modifiedDate: '2022-11-03T18:26:30.477Z',
    version: 3,
    policy: {
      phases: {
        warm: {
          min_age: '4d',
          actions: { forcemerge: { max_num_segments: 1 }, set_priority: { priority: 50 } },
        },
        cold: { min_age: '90d', actions: { set_priority: { priority: 0 } } },
        hot: {
          min_age: '0ms',
          actions: { rollover: { max_primary_shard_size: '50gb', max_age: '3d' } },
        },
        delete: { min_age: '180d', actions: { delete: { delete_searchable_snapshot: true } } },
      },
    },
    indices: [],
    dataStreams: [],
    indexTemplates: ['synthetics'],
  },
  {
    name: 'synthetics-synthetics.browser-default_policy',
    modifiedDate: '2022-10-10T16:22:38.714Z',
    version: 1,
    policy: {
      phases: {
        hot: {
          min_age: '0ms',
          actions: {
            rollover: { max_size: '50gb', max_age: '30d' },
            set_priority: { priority: 100 },
          },
        },
        delete: { min_age: '365d', actions: { delete: { delete_searchable_snapshot: true } } },
      },
      _meta: { package: { name: 'synthetics' }, managed_by: 'fleet', managed: true },
    },
    indices: [
      '.ds-synthetics-browser-default-2022.10.11-000001',
      '.ds-synthetics-browser-test_monitor-2022.10.13-000001',
      '.ds-synthetics-browser-test_monitor-2022.11.01-000002',
      '.ds-synthetics-browser-default-2022.11.01-000002',
    ],
    dataStreams: ['synthetics-browser-default', 'synthetics-browser-test_monitor'],
    indexTemplates: ['synthetics-browser'],
  },
  {
    name: 'synthetics-synthetics.browser_network-default_policy',
    modifiedDate: '2022-11-01T07:11:11.452Z',
    version: 2,
    policy: {
      phases: {
        hot: {
          min_age: '0ms',
          actions: { rollover: { max_age: '1d' }, set_priority: { priority: 100 } },
        },
        delete: { min_age: '14d', actions: { delete: { delete_searchable_snapshot: true } } },
      },
      _meta: { package: { name: 'synthetics' }, managed_by: 'fleet', managed: true },
    },
    indices: [
      '.ds-synthetics-browser.network-default-2022.10.31-000022',
      '.ds-synthetics-browser.network-default-2022.10.26-000017',
      '.ds-synthetics-browser.network-default-2022.11.04-000030',
      '.ds-synthetics-browser.network-test_monitor-2022.10.14-000002',
      '.ds-synthetics-browser.network-default-2022.11.07-000036',
      '.ds-synthetics-browser.network-default-2022.11.05-000032',
      '.ds-synthetics-browser.network-default-2022.11.08-000038',
      '.ds-synthetics-browser.network-default-2022.11.06-000034',
      '.ds-synthetics-browser.network-default-2022.10.25-000015',
      '.ds-synthetics-browser.network-default-2022.11.02-000026',
      '.ds-synthetics-browser.network-default-2022.11.01-000024',
      '.ds-synthetics-browser.network-default-2022.11.03-000028',
    ],
    dataStreams: ['synthetics-browser.network-test_monitor', 'synthetics-browser.network-default'],
    indexTemplates: ['synthetics-browser.network'],
  },
  {
    name: 'synthetics-synthetics.browser_screenshot-default_policy',
    modifiedDate: '2022-11-03T20:43:55.861Z',
    version: 2,
    policy: {
      phases: {
        hot: {
          min_age: '0ms',
          actions: { rollover: { max_age: '1d' }, set_priority: { priority: 100 } },
        },
        delete: { min_age: '14d', actions: { delete: { delete_searchable_snapshot: true } } },
      },
      _meta: { package: { name: 'synthetics' }, managed_by: 'fleet', managed: true },
    },
    indices: [
      '.ds-synthetics-browser.screenshot-default-2022.11.05-000031',
      '.ds-synthetics-browser.screenshot-default-2022.11.06-000033',
      '.ds-synthetics-browser.screenshot-default-2022.11.08-000036',
      '.ds-synthetics-browser.screenshot-default-2022.11.07-000034',
      '.ds-synthetics-browser.screenshot-default-2022.11.01-000023',
      '.ds-synthetics-browser.screenshot-default-2022.11.02-000025',
      '.ds-synthetics-browser.screenshot-default-2022.11.03-000027',
      '.ds-synthetics-browser.screenshot-default-2022.11.04-000029',
      '.ds-synthetics-browser.screenshot-default-2022.10.25-000014',
      '.ds-synthetics-browser.screenshot-default-2022.10.31-000021',
      '.ds-synthetics-browser.screenshot-test_monitor-2022.10.14-000002',
      '.ds-synthetics-browser.screenshot-default-2022.10.26-000015',
    ],
    dataStreams: [
      'synthetics-browser.screenshot-default',
      'synthetics-browser.screenshot-test_monitor',
    ],
    indexTemplates: ['synthetics-browser.screenshot'],
  },
  {
    name: 'synthetics-synthetics.http-default_policy',
    modifiedDate: '2022-11-03T20:43:55.779Z',
    version: 2,
    policy: {
      phases: {
        hot: {
          min_age: '0ms',
          actions: {
            rollover: { max_size: '50gb', max_age: '30d' },
            set_priority: { priority: 100 },
          },
        },
        delete: { min_age: '365d', actions: { delete: { delete_searchable_snapshot: true } } },
      },
      _meta: { package: { name: 'synthetics' }, managed_by: 'fleet', managed: true },
    },
    indices: [
      '.ds-synthetics-http-default-2022.10.10-000001',
      '.ds-synthetics-http-default-2022.11.01-000002',
    ],
    dataStreams: ['synthetics-http-default'],
    indexTemplates: ['synthetics-http'],
  },
  {
    name: 'synthetics-synthetics.icmp-default_policy',
    modifiedDate: '2022-11-03T20:43:55.819Z',
    version: 1,
    policy: {
      phases: {
        hot: {
          min_age: '0ms',
          actions: {
            rollover: { max_size: '50gb', max_age: '30d' },
            set_priority: { priority: 100 },
          },
        },
        delete: { min_age: '365d', actions: { delete: { delete_searchable_snapshot: true } } },
      },
      _meta: { package: { name: 'synthetics' }, managed_by: 'fleet', managed: true },
    },
    indices: [],
    dataStreams: [],
    indexTemplates: ['synthetics-icmp'],
  },
  {
    name: 'synthetics-synthetics.tcp-default_policy',
    modifiedDate: '2022-11-03T20:43:55.911Z',
    version: 1,
    policy: {
      phases: {
        hot: {
          min_age: '0ms',
          actions: {
            rollover: { max_size: '50gb', max_age: '30d' },
            set_priority: { priority: 100 },
          },
        },
        delete: { min_age: '365d', actions: { delete: { delete_searchable_snapshot: true } } },
      },
      _meta: { package: { name: 'synthetics' }, managed_by: 'fleet', managed: true },
    },
    indices: [],
    dataStreams: [],
    indexTemplates: ['synthetics-tcp'],
  },
];

const indexSize = {
  data: [
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.screenshot-default-2022.11.05-000031',
      uuid: 'oCENP4X0SVGpk0PiX4hHRg',
      pri: '1',
      rep: '1',
      'docs.count': '145',
      'docs.deleted': '0',
      'store.size': '665872',
      'pri.store.size': '332936',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.network-default-2022.10.26-000017',
      uuid: 'beBLW28lTTCn2ewlgMnVqQ',
      pri: '1',
      rep: '1',
      'docs.count': '857',
      'docs.deleted': '0',
      'store.size': '3203973',
      'pri.store.size': '1683610',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.screenshot-default-2022.11.04-000029',
      uuid: 'cha8c9z2RnSwE38aIqDAtg',
      pri: '1',
      rep: '1',
      'docs.count': '591',
      'docs.deleted': '0',
      'store.size': '2260733',
      'pri.store.size': '1122727',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.screenshot-default-2022.11.06-000033',
      uuid: 'cXTFRYL5RzuVX0K8X-Neyg',
      pri: '1',
      rep: '1',
      'docs.count': '434',
      'docs.deleted': '0',
      'store.size': '1628330',
      'pri.store.size': '803915',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-http-default-2022.10.10-000001',
      uuid: 'MAd6ZMFmSBGu_GUhQbYNWQ',
      pri: '1',
      rep: '1',
      'docs.count': '3627',
      'docs.deleted': '0',
      'store.size': '27561629',
      'pri.store.size': '14524475',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.network-default-2022.11.02-000026',
      uuid: 'akEmHfQZTHaWSt3VmRvWwA',
      pri: '1',
      rep: '1',
      'docs.count': '23217',
      'docs.deleted': '0',
      'store.size': '53762582',
      'pri.store.size': '26520951',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.screenshot-default-2022.10.31-000021',
      uuid: '2pfVA_tYTaqFi1yGYD8sJg',
      pri: '1',
      rep: '1',
      'docs.count': '519',
      'docs.deleted': '0',
      'store.size': '1860519',
      'pri.store.size': '921432',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.network-default-2022.11.03-000028',
      uuid: '5Xl38ooiQb6ex8-S9KyrnA',
      pri: '1',
      rep: '1',
      'docs.count': '20237',
      'docs.deleted': '0',
      'store.size': '46353084',
      'pri.store.size': '22833359',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.network-default-2022.11.05-000032',
      uuid: 'md-p7BxQTdyBoQaOeLfFIQ',
      pri: '1',
      rep: '1',
      'docs.count': '2210',
      'docs.deleted': '0',
      'store.size': '7450666',
      'pri.store.size': '3725333',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser-default-2022.11.01-000002',
      uuid: 'ndlhAGIAQcqKQD69vfPLow',
      pri: '1',
      rep: '1',
      'docs.count': '19730',
      'docs.deleted': '0',
      'store.size': '12851228',
      'pri.store.size': '6528686',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.network-default-2022.11.06-000034',
      uuid: 'CbrdjohORn-WiSshOCkrSA',
      pri: '1',
      rep: '1',
      'docs.count': '10486',
      'docs.deleted': '0',
      'store.size': '24675440',
      'pri.store.size': '12352425',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.screenshot-default-2022.11.07-000034',
      uuid: 'pA7oRalRRgGmQx_v6kGlnw',
      pri: '1',
      rep: '1',
      'docs.count': '514',
      'docs.deleted': '0',
      'store.size': '1819473',
      'pri.store.size': '882965',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-http-default-2022.11.01-000002',
      uuid: '6RTauuM8TRGhJc1N8ZZzyQ',
      pri: '1',
      rep: '1',
      'docs.count': '2466',
      'docs.deleted': '0',
      'store.size': '10238049',
      'pri.store.size': '5430978',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.screenshot-default-2022.11.08-000036',
      uuid: 'qukXPoRtRXuOLdAIR9xJVw',
      pri: '1',
      rep: '1',
      'docs.count': '516',
      'docs.deleted': '0',
      'store.size': '1840166',
      'pri.store.size': '889039',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.network-default-2022.10.31-000022',
      uuid: 'xLlWgjyyRv-OVsBKbP01Vg',
      pri: '1',
      rep: '1',
      'docs.count': '7129',
      'docs.deleted': '0',
      'store.size': '18161661',
      'pri.store.size': '8881398',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser-test_monitor-2022.11.01-000002',
      uuid: 'RTxZocsMSGqcKTQkgLCRvg',
      pri: '1',
      rep: '1',
      'docs.count': '0',
      'docs.deleted': '0',
      'store.size': '450',
      'pri.store.size': '225',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.screenshot-default-2022.10.25-000014',
      uuid: 'XKCWQwG_Rm6d9l0Y5lv3cw',
      pri: '1',
      rep: '1',
      'docs.count': '606',
      'docs.deleted': '0',
      'store.size': '1755495',
      'pri.store.size': '892487',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.network-default-2022.11.07-000036',
      uuid: 'Ptjm8e5dS5ahgdqx3KbfcQ',
      pri: '1',
      rep: '1',
      'docs.count': '15515',
      'docs.deleted': '0',
      'store.size': '32981897',
      'pri.store.size': '16160933',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.screenshot-default-2022.11.01-000023',
      uuid: 'Nj2eSpTdRViXxjGgR6QDRw',
      pri: '1',
      rep: '1',
      'docs.count': '1066',
      'docs.deleted': '0',
      'store.size': '3891146',
      'pri.store.size': '1907349',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.network-default-2022.11.08-000038',
      uuid: 'QlKLepPXTAWxBosM4X6elg',
      pri: '1',
      rep: '1',
      'docs.count': '13814',
      'docs.deleted': '0',
      'store.size': '29443605',
      'pri.store.size': '14997832',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser-test_monitor-2022.10.13-000001',
      uuid: '0tOfPYLTQ3Opo5X-aH0qEg',
      pri: '1',
      rep: '1',
      'docs.count': '104',
      'docs.deleted': '0',
      'store.size': '530788',
      'pri.store.size': '280950',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.network-default-2022.10.25-000015',
      uuid: 'IW_ud5eZQ6aN_yd5Y78C7g',
      pri: '1',
      rep: '1',
      'docs.count': '9863',
      'docs.deleted': '0',
      'store.size': '24126155',
      'pri.store.size': '11436675',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.network-test_monitor-2022.10.14-000002',
      uuid: 'qb3UNKLQSIKoqO3e4mzscg',
      pri: '1',
      rep: '1',
      'docs.count': '0',
      'docs.deleted': '0',
      'store.size': '450',
      'pri.store.size': '225',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.network-default-2022.11.01-000024',
      uuid: 'zKPwzZR7Q2yDau1SStRRpQ',
      pri: '1',
      rep: '1',
      'docs.count': '29064',
      'docs.deleted': '0',
      'store.size': '64835940',
      'pri.store.size': '33251107',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.screenshot-default-2022.10.26-000015',
      uuid: 'Pf5SMZw9T-CHk2G2tV600g',
      pri: '1',
      rep: '1',
      'docs.count': '110',
      'docs.deleted': '0',
      'store.size': '440107',
      'pri.store.size': '209649',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.network-default-2022.11.04-000030',
      uuid: 'EyGQha6dTC-nQ2SiRNAi9A',
      pri: '1',
      rep: '1',
      'docs.count': '12872',
      'docs.deleted': '0',
      'store.size': '32964903',
      'pri.store.size': '16163252',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.screenshot-default-2022.11.02-000025',
      uuid: 'FeIPo9Q7QS2JXqeuTobXBg',
      pri: '1',
      rep: '1',
      'docs.count': '976',
      'docs.deleted': '0',
      'store.size': '3309415',
      'pri.store.size': '1686618',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.screenshot-test_monitor-2022.10.14-000002',
      uuid: 'OxDJsEGaQaKASYXcmEUYNg',
      pri: '1',
      rep: '1',
      'docs.count': '0',
      'docs.deleted': '0',
      'store.size': '450',
      'pri.store.size': '225',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser-default-2022.10.11-000001',
      uuid: 'Pcud3OXiRguEBf0xBEMX2w',
      pri: '1',
      rep: '1',
      'docs.count': '61934',
      'docs.deleted': '0',
      'store.size': '44228681',
      'pri.store.size': '21588082',
    },
    {
      health: 'green',
      status: 'open',
      index: '.ds-synthetics-browser.screenshot-default-2022.11.03-000027',
      uuid: 'C3RUyZ7TTgqihSIjPu_bVA',
      pri: '1',
      rep: '1',
      'docs.count': '761',
      'docs.deleted': '0',
      'store.size': '2449941',
      'pri.store.size': '1265991',
    },
  ],
  _inspect: [],
};

const getTestData = () => {
  return { policiesData, indexSize };
};
